@@ -1,17 +1,15 @@
-from __future__ import print_function, absolute_import
 import numpy as np
 from numba import cuda
-from numba.cuda.testing import unittest, SerialMixin
+from numba.cuda.testing import unittest, CUDATestCase
 
 
-class TestCudaComplex(SerialMixin, unittest.TestCase):
+class TestCudaComplex(CUDATestCase):
     def test_cuda_complex_arg(self):
         @cuda.jit('void(complex128[:], complex128)')
         def foo(a, b):
             i = cuda.grid(1)
             a[i] += b
 
-
         a = np.arange(5, dtype=np.complex128)
         a0 = a.copy()
         foo[1, a.shape](a, 2j)
@@ -20,5 +18,3 @@ class TestCudaComplex(SerialMixin, unittest.TestCase):
 
 if __name__ == '__main__':
     unittest.main()
-
-
