@@ -1,11 +1,34 @@
 import numpy as np
 import numpy.core.umath_tests as ut
 
+from collections import namedtuple
 from numba import void, float32, float64
 from numba import guvectorize
 from numba import cuda
 from numba.cuda.testing import skip_on_cudasim, CUDATestCase
 import unittest
+import warnings
+from numba.core.errors import NumbaPerformanceWarning
+from numba.tests.support import override_config
+
+
+def _get_matmulcore_gufunc(dtype=float32, max_blocksize=None):
+    @guvectorize([void(dtype[:, :], dtype[:, :], dtype[:, :])],
+                 '(m,n),(n,p)->(m,p)',
+                 target='cuda')
+    def matmulcore(A, B, C):
+        m, n = A.shape
+        n, p = B.shape
+        for i in range(m):
+            for j in range(p):
+                C[i, j] = 0
+                for k in range(n):
+                    C[i, j] += A[i, k] * B[k, j]
+
+    gufunc = matmulcore
+    if max_blocksize:
+        gufunc.max_blocksize = max_blocksize
+    return gufunc
 
 
 @skip_on_cudasim('ufunc API unsupported in the simulator')
@@ -13,20 +36,7 @@ class TestCUDAGufunc(CUDATestCase):
 
     def test_gufunc_small(self):
 
-        @guvectorize([void(float32[:, :], float32[:, :], float32[:, :])],
-                     '(m,n),(n,p)->(m,p)',
-                     target='cuda')
-        def matmulcore(A, B, C):
-            m, n = A.shape
-            n, p = B.shape
-            for i in range(m):
-                for j in range(p):
-                    C[i, j] = 0
-                    for k in range(n):
-                        C[i, j] += A[i, k] * B[k, j]
-
-        gufunc = matmulcore
-        gufunc.max_blocksize = 512
+        gufunc = _get_matmulcore_gufunc(max_blocksize=512)
 
         matrix_ct = 2
         A = np.arange(matrix_ct * 2 * 4, dtype=np.float32).reshape(matrix_ct, 2,
@@ -40,20 +50,7 @@ class TestCUDAGufunc(CUDATestCase):
 
     def test_gufunc_auto_transfer(self):
 
-        @guvectorize([void(float32[:, :], float32[:, :], float32[:, :])],
-                     '(m,n),(n,p)->(m,p)',
-                     target='cuda')
-        def matmulcore(A, B, C):
-            m, n = A.shape
-            n, p = B.shape
-            for i in range(m):
-                for j in range(p):
-                    C[i, j] = 0
-                    for k in range(n):
-                        C[i, j] += A[i, k] * B[k, j]
-
-        gufunc = matmulcore
-        gufunc.max_blocksize = 512
+        gufunc = _get_matmulcore_gufunc(max_blocksize=512)
 
         matrix_ct = 2
         A = np.arange(matrix_ct * 2 * 4, dtype=np.float32).reshape(matrix_ct, 2,
@@ -69,20 +66,7 @@ class TestCUDAGufunc(CUDATestCase):
 
     def test_gufunc(self):
 
-        @guvectorize([void(float32[:, :], float32[:, :], float32[:, :])],
-                     '(m,n),(n,p)->(m,p)',
-                     target='cuda')
-        def matmulcore(A, B, C):
-            m, n = A.shape
-            n, p = B.shape
-            for i in range(m):
-                for j in range(p):
-                    C[i, j] = 0
-                    for k in range(n):
-                        C[i, j] += A[i, k] * B[k, j]
-
-        gufunc = matmulcore
-        gufunc.max_blocksize = 512
+        gufunc = _get_matmulcore_gufunc(max_blocksize=512)
 
         matrix_ct = 1001 # an odd number to test thread/block division in CUDA
         A = np.arange(matrix_ct * 2 * 4, dtype=np.float32).reshape(matrix_ct, 2,
@@ -96,20 +80,7 @@ class TestCUDAGufunc(CUDATestCase):
 
     def test_gufunc_hidim(self):
 
-        @guvectorize([void(float32[:, :], float32[:, :], float32[:, :])],
-                     '(m,n),(n,p)->(m,p)',
-                     target='cuda')
-        def matmulcore(A, B, C):
-            m, n = A.shape
-            n, p = B.shape
-            for i in range(m):
-                for j in range(p):
-                    C[i, j] = 0
-                    for k in range(n):
-                        C[i, j] += A[i, k] * B[k, j]
-
-        gufunc = matmulcore
-        gufunc.max_blocksize = 512
+        gufunc = _get_matmulcore_gufunc(max_blocksize=512)
 
         matrix_ct = 100 # an odd number to test thread/block division in CUDA
         A = np.arange(matrix_ct * 2 * 4, dtype=np.float32).reshape(4, 25, 2, 4)
@@ -121,19 +92,7 @@ class TestCUDAGufunc(CUDATestCase):
 
     def test_gufunc_new_axis(self):
 
-        @guvectorize([void(float64[:, :], float64[:, :], float64[:, :])],
-                     '(m,n),(n,p)->(m,p)',
-                     target='cuda')
-        def matmulcore(A, B, C):
-            m, n = A.shape
-            n, p = B.shape
-            for i in range(m):
-                for j in range(p):
-                    C[i, j] = 0
-                    for k in range(n):
-                        C[i, j] += A[i, k] * B[k, j]
-
-        gufunc = matmulcore
+        gufunc = _get_matmulcore_gufunc(dtype=float64)
 
         X = np.random.randn(10, 3, 3)
         Y = np.random.randn(3, 3)
@@ -148,20 +107,7 @@ class TestCUDAGufunc(CUDATestCase):
 
     def test_gufunc_adjust_blocksize(self):
 
-        @guvectorize([void(float32[:, :], float32[:, :], float32[:, :])],
-                     '(m,n),(n,p)->(m,p)',
-                     target='cuda')
-        def matmulcore(A, B, C):
-            m, n = A.shape
-            n, p = B.shape
-            for i in range(m):
-                for j in range(p):
-                    C[i, j] = 0
-                    for k in range(n):
-                        C[i, j] += A[i, k] * B[k, j]
-
-        gufunc = matmulcore
-        gufunc.max_blocksize = 512
+        gufunc = _get_matmulcore_gufunc(max_blocksize=512)
 
         matrix_ct = 1001 # an odd number to test thread/block division in CUDA
         A = np.arange(matrix_ct * 2 * 4, dtype=np.float32).reshape(matrix_ct, 2,
@@ -176,20 +122,7 @@ class TestCUDAGufunc(CUDATestCase):
 
     def test_gufunc_stream(self):
 
-        @guvectorize([void(float32[:, :], float32[:, :], float32[:, :])],
-                     '(m,n),(n,p)->(m,p)',
-                     target='cuda')
-        def matmulcore(A, B, C):
-            m, n = A.shape
-            n, p = B.shape
-            for i in range(m):
-                for j in range(p):
-                    C[i, j] = 0
-                    for k in range(n):
-                        C[i, j] += A[i, k] * B[k, j]
-
-        gufunc = matmulcore
-        gufunc.max_blocksize = 512
+        gufunc = _get_matmulcore_gufunc(max_blocksize=512)
 
         #cuda.driver.flush_pending_free()
         matrix_ct = 1001 # an odd number to test thread/block division in CUDA
@@ -254,6 +187,46 @@ class TestCUDAGufunc(CUDATestCase):
         copy2d(A, out=B)
         self.assertTrue(np.allclose(A, B))
 
+    # Test inefficient use of the GPU where the inputs are all mapped onto a
+    # single thread in a single block.
+    def test_inefficient_launch_configuration(self):
+        @guvectorize(['void(float32[:], float32[:], float32[:])'],
+                     '(n),(n)->(n)', target='cuda')
+        def numba_dist_cuda(a, b, dist):
+            len = a.shape[0]
+            for i in range(len):
+                dist[i] = a[i] * b[i]
+
+        a = np.random.rand(1024 * 32).astype('float32')
+        b = np.random.rand(1024 * 32).astype('float32')
+        dist = np.zeros(a.shape[0]).astype('float32')
+
+        with override_config('CUDA_LOW_OCCUPANCY_WARNINGS', 1):
+            with warnings.catch_warnings(record=True) as w:
+                numba_dist_cuda(a, b, dist)
+                self.assertEqual(w[0].category, NumbaPerformanceWarning)
+                self.assertIn('Grid size', str(w[0].message))
+                self.assertIn('low occupancy', str(w[0].message))
+
+    def test_efficient_launch_configuration(self):
+        @guvectorize(['void(float32[:], float32[:], float32[:])'],
+                     '(n),(n)->(n)', nopython=True, target='cuda')
+        def numba_dist_cuda2(a, b, dist):
+            len = a.shape[0]
+            for i in range(len):
+                dist[i] = a[i] * b[i]
+
+        a = np.random.rand(524288 * 2).astype('float32').\
+            reshape((524288, 2))
+        b = np.random.rand(524288 * 2).astype('float32').\
+            reshape((524288, 2))
+        dist = np.zeros_like(a)
+
+        with override_config('CUDA_LOW_OCCUPANCY_WARNINGS', 1):
+            with warnings.catch_warnings(record=True) as w:
+                numba_dist_cuda2(a, b, dist)
+                self.assertEqual(len(w), 0)
+
     def test_nopython_flag(self):
 
         def foo(A, B):
@@ -296,6 +269,41 @@ class TestCUDAGufunc(CUDATestCase):
         msg = "cannot specify 'out' as both a positional and keyword argument"
         self.assertEqual(str(raises.exception), msg)
 
+    def check_tuple_arg(self, a, b):
+        @guvectorize([(float64[:], float64[:], float64[:])], '(n),(n)->()',
+                     target='cuda')
+        def gu_reduce(x, y, r):
+            s = 0
+            for i in range(len(x)):
+                s += x[i] * y[i]
+            r[0] = s
+
+        r = gu_reduce(a, b)
+        expected = np.sum(np.asarray(a) * np.asarray(b), axis=1)
+        np.testing.assert_equal(expected, r)
+
+    def test_tuple_of_tuple_arg(self):
+        a = ((1.0, 2.0, 3.0),
+             (4.0, 5.0, 6.0))
+        b = ((1.5, 2.5, 3.5),
+             (4.5, 5.5, 6.5))
+        self.check_tuple_arg(a, b)
+
+    def test_tuple_of_namedtuple_arg(self):
+        Point = namedtuple('Point', ('x', 'y', 'z'))
+        a = (Point(x=1.0, y=2.0, z=3.0),
+             Point(x=4.0, y=5.0, z=6.0))
+        b = (Point(x=1.5, y=2.5, z=3.5),
+             Point(x=4.5, y=5.5, z=6.5))
+        self.check_tuple_arg(a, b)
+
+    def test_tuple_of_array_arg(self):
+        a = (np.asarray((1.0, 2.0, 3.0)),
+             np.asarray((4.0, 5.0, 6.0)))
+        b = (np.asarray((1.5, 2.5, 3.5)),
+             np.asarray((4.5, 5.5, 6.5)))
+        self.check_tuple_arg(a, b)
+
 
 if __name__ == '__main__':
     unittest.main()
