@@ -1,4 +1,5 @@
 from itertools import product, cycle, permutations
+import gc
 import sys
 import warnings
 
@@ -7,8 +8,8 @@ import numpy as np
 from numba import jit, typeof
 from numba.core import types
 from numba.core.compiler import compile_isolated
-from numba.core.errors import TypingError, LoweringError
-from numba.np.numpy_support import as_dtype
+from numba.core.errors import TypingError, LoweringError, NumbaValueError
+from numba.np.numpy_support import as_dtype, numpy_version
 from numba.tests.support import (TestCase, CompilationCache, MemoryLeak,
                                  MemoryLeakMixin, tag, needs_blas)
 import unittest
@@ -232,6 +233,24 @@ def array_real(a):
 def array_imag(a):
     return np.imag(a)
 
+def np_clip_no_out(a, a_min, a_max):
+    return np.clip(a, a_min, a_max)
+
+def np_clip(a, a_min, a_max, out=None):
+    return np.clip(a, a_min, a_max, out)
+
+def np_clip_kwargs(a, a_min, a_max, out=None):
+    return np.clip(a, a_min, a_max, out=out)
+
+def array_clip(a, a_min=None, a_max=None, out=None):
+    return a.clip(a_min, a_max, out)
+
+def array_clip_kwargs(a, a_min=None, a_max=None, out=None):
+    return a.clip(a_min, a_max, out=out)
+
+def array_clip_no_out(a, a_min, a_max):
+    return a.clip(a_min, a_max)
+
 def array_conj(a):
     return a.conj()
 
@@ -251,7 +270,6 @@ def array_dot_chain(a, b):
 def array_ctor(n, dtype):
     return np.ones(n, dtype=dtype)
 
-
 class TestArrayMethods(MemoryLeakMixin, TestCase):
     """
     Test various array methods and array-related functions.
@@ -365,11 +383,73 @@ class TestArrayMethods(MemoryLeakMixin, TestCase):
                 run(arr, dtype)
             self.assertEqual(str(raises.exception),
                              "new type not compatible with array")
+        def check_err_noncontig_last_axis(arr, dtype):
+            # check NumPy interpreted version raises
+            msg = ("To change to a dtype of a different size, the last axis "
+                   "must be contiguous")
+            with self.assertRaises(ValueError) as raises:
+                make_array_view(dtype)(arr)
+            self.assertEqual(str(raises.exception), msg)
+            # check Numba version raises
+            with self.assertRaises(ValueError) as raises:
+                run(arr, dtype)
+            self.assertEqual(str(raises.exception), msg)
+
+        def check_err_0d(arr, dtype):
+            # check NumPy interpreted version raises
+            msg = ("Changing the dtype of a 0d array is only supported "
+                   "if the itemsize is unchanged")
+            with self.assertRaises(ValueError) as raises:
+                make_array_view(dtype)(arr)
+            self.assertEqual(str(raises.exception), msg)
+            # check Numba version raises
+            with self.assertRaises(ValueError) as raises:
+                run(arr, dtype)
+            self.assertEqual(str(raises.exception), msg)
+
+        def check_err_smaller_dtype(arr, dtype):
+            # check NumPy interpreted version raises
+            msg = ("When changing to a smaller dtype, its size must be a "
+                   "divisor of the size of original dtype")
+            with self.assertRaises(ValueError) as raises:
+                make_array_view(dtype)(arr)
+            self.assertEqual(str(raises.exception), msg)
+            # check Numba version raises
+            with self.assertRaises(ValueError) as raises:
+                run(arr, dtype)
+            self.assertEqual(str(raises.exception), msg)
+
+        def check_err_larger_dtype(arr, dtype):
+            # check NumPy interpreted version raises
+            msg = ("When changing to a larger dtype, its size must be a "
+                   "divisor of the total size in bytes of the last axis "
+                   "of the array.")
+            with self.assertRaises(ValueError) as raises:
+                make_array_view(dtype)(arr)
+            self.assertEqual(str(raises.exception), msg)
+            # check Numba version raises
+            with self.assertRaises(ValueError) as raises:
+                run(arr, dtype)
+            self.assertEqual(str(raises.exception), msg)
 
         dt1 = np.dtype([('a', np.int8), ('b', np.int8)])
         dt2 = np.dtype([('u', np.int16), ('v', np.int8)])
         dt3 = np.dtype([('x', np.int16), ('y', np.int16)])
 
+        # The checking routines are much more specific from NumPy 1.23 onwards
+        # as the granularity of error reporing is improved in Numba to match
+        # that of NumPy.
+        if numpy_version >= (1, 23):
+            check_error_larger_dt = check_err_larger_dtype
+            check_error_smaller_dt = check_err_smaller_dtype
+            check_error_noncontig = check_err_noncontig_last_axis
+            check_error_0d = check_err_0d
+        else:
+            check_error_larger_dt = check_err
+            check_error_smaller_dt = check_err
+            check_error_noncontig = check_err
+            check_error_0d = check_err
+
         # C-contiguous
         arr = np.arange(24, dtype=np.int8)
         check(arr, np.dtype('int16'))
@@ -379,7 +459,7 @@ class TestArrayMethods(MemoryLeakMixin, TestCase):
         check(arr, np.complex64)
         check(arr, dt1)
         check(arr, dt2)
-        check_err(arr, np.complex128)
+        check_error_larger_dt(arr, np.complex128)
 
         # Last dimension must have a compatible size
         arr = arr.reshape((3, 8))
@@ -387,39 +467,56 @@ class TestArrayMethods(MemoryLeakMixin, TestCase):
         check(arr, np.float32)
         check(arr, np.complex64)
         check(arr, dt1)
-        check_err(arr, dt2)
-        check_err(arr, np.complex128)
+        check_error_larger_dt(arr, dt2)
+        check_error_larger_dt(arr, np.complex128)
 
         # F-contiguous
-        arr = np.arange(24, dtype=np.int8).reshape((3, 8)).T
-        check(arr, np.int8)
-        check(arr, np.float32)
-        check(arr, np.complex64)
-        check(arr, dt1)
-        check_err(arr, dt2)
-        check_err(arr, np.complex128)
+        f_arr = np.arange(24, dtype=np.int8).reshape((3, 8)).T
+        # neither F or C contiguous
+        not_f_or_c_arr = np.zeros((4, 4)).T[::2, ::2]
+
+        # NumPy 1.23 does not allow views with different size dtype for
+        # non-contiguous last axis.
+        if numpy_version >= (1, 23):
+            check_maybe_error = check_err_noncontig_last_axis
+        else:
+            check_maybe_error = check
+
+        check(f_arr, np.int8)
+        check(not_f_or_c_arr, np.uint64)
+        check_maybe_error(f_arr, np.float32)
+        check_maybe_error(f_arr, np.complex64)
+        check_maybe_error(f_arr, dt1)
+
+        check_error_noncontig(f_arr, dt2)
+        check_error_noncontig(f_arr, np.complex128)
+        check_error_noncontig(not_f_or_c_arr, np.int8)
 
         # Non-contiguous: only a type with the same itemsize can be used
         arr = np.arange(16, dtype=np.int32)[::2]
         check(arr, np.uint32)
         check(arr, np.float32)
         check(arr, dt3)
-        check_err(arr, np.int8)
-        check_err(arr, np.int16)
-        check_err(arr, np.int64)
-        check_err(arr, dt1)
-        check_err(arr, dt2)
+        check_error_noncontig(arr, np.int8)
+        check_error_noncontig(arr, np.int16)
+        check_error_noncontig(arr, np.int64)
+        check_error_noncontig(arr, dt1)
+        check_error_noncontig(arr, dt2)
 
-        # Zero-dim array: only a type with the same itemsize can be used
+        ## Zero-dim array: only a type with the same itemsize can be used
         arr = np.array([42], dtype=np.int32).reshape(())
         check(arr, np.uint32)
         check(arr, np.float32)
         check(arr, dt3)
-        check_err(arr, np.int8)
-        check_err(arr, np.int16)
-        check_err(arr, np.int64)
-        check_err(arr, dt1)
-        check_err(arr, dt2)
+        check_error_0d(arr, np.int8)
+        check_error_0d(arr, np.int16)
+        check_error_0d(arr, np.int64)
+        check_error_0d(arr, dt1)
+        check_error_0d(arr, dt2)
+
+        # Changing to smaller dtype
+        arr = np.array(['abcdef'])
+        check_error_smaller_dt(arr, np.complex128)
 
         # Exceptions leak references
         self.disable_leak_check()
@@ -457,6 +554,7 @@ class TestArrayMethods(MemoryLeakMixin, TestCase):
         check(arr, np.int32)
         check(arr, np.float32)
         check(arr, np.complex128)
+        check(arr, "float32")
 
         # F-contiguous
         arr = np.arange(24, dtype=np.int8).reshape((3, 8)).T
@@ -477,6 +575,15 @@ class TestArrayMethods(MemoryLeakMixin, TestCase):
             check(arr, dt)
         self.assertIn('cannot convert from int32 to Record',
                       str(raises.exception))
+        # Check non-Literal string raises
+        unicode_val = "float32"
+        with self.assertTypingError() as raises:
+            @jit(nopython=True)
+            def foo(dtype):
+                np.array([1]).astype(dtype)
+            foo(unicode_val)
+        self.assertIn('array.astype if dtype is a string it must be constant',
+                      str(raises.exception))
 
     def check_np_frombuffer(self, pyfunc):
         def run(buf):
@@ -489,8 +596,12 @@ class TestArrayMethods(MemoryLeakMixin, TestCase):
             got = run(buf)
             self.assertPreciseEqual(got, expected)
             del expected
+            # Note gc.collect is due to references in `except ... as e` that
+            # aren't immediately cleared
+            gc.collect()
             self.assertEqual(sys.getrefcount(buf), old_refcnt + 1)
             del got
+            gc.collect()
             self.assertEqual(sys.getrefcount(buf), old_refcnt)
             self.memory_leak_teardown()
 
@@ -564,11 +675,22 @@ class TestArrayMethods(MemoryLeakMixin, TestCase):
     def test_np_copy(self):
         self.check_layout_dependent_func(np_copy)
 
+    def check_ascontiguousarray_scalar(self, pyfunc):
+        def check_scalar(x):
+            cres = compile_isolated(pyfunc, (typeof(x), ))
+            expected = pyfunc(x)
+            got = cres.entry_point(x)
+            self.assertPreciseEqual(expected, got)
+        for x in [42, 42.0, 42j, np.float32(42), np.float64(42), True]:
+            check_scalar(x)
+
     def test_np_asfortranarray(self):
         self.check_layout_dependent_func(np_asfortranarray)
+        self.check_ascontiguousarray_scalar(np_asfortranarray)
 
     def test_np_ascontiguousarray(self):
         self.check_layout_dependent_func(np_ascontiguousarray)
+        self.check_ascontiguousarray_scalar(np_ascontiguousarray)
 
     def check_np_frombuffer_allocated(self, pyfunc):
         def run(shape):
@@ -785,7 +907,7 @@ class TestArrayMethods(MemoryLeakMixin, TestCase):
             check_ok(4)
             check_ok(5.5)
             check_ok(-3)
-            check_ok(np.complex(4, 4))
+            check_ok(complex(4, 4))
             check_ok(np.int8(0))
 
     def test_arange_2_arg(self):
@@ -810,8 +932,8 @@ class TestArrayMethods(MemoryLeakMixin, TestCase):
             check_ok(-8, -1, pyfunc, cfunc)
             check_ok(4, 0.5, pyfunc, cfunc)
             check_ok(0.5, 4, pyfunc, cfunc)
-            check_ok(np.complex(1, 1), np.complex(4, 4), pyfunc, cfunc)
-            check_ok(np.complex(4, 4), np.complex(1, 1), pyfunc, cfunc)
+            check_ok(complex(1, 1), complex(4, 4), pyfunc, cfunc)
+            check_ok(complex(4, 4), complex(1, 1), pyfunc, cfunc)
             check_ok(3, None, pyfunc, cfunc)
 
         pyfunc = np_arange_1_dtype
@@ -845,7 +967,7 @@ class TestArrayMethods(MemoryLeakMixin, TestCase):
             check_ok(0, -10, -2, pyfunc, cfunc)
             check_ok(0.5, 4, 2, pyfunc, cfunc)
             check_ok(0, 1, 0.1, pyfunc, cfunc)
-            check_ok(0, np.complex(4, 4), np.complex(1, 1), pyfunc, cfunc)
+            check_ok(0, complex(4, 4), complex(1, 1), pyfunc, cfunc)
             check_ok(3, 6, None, pyfunc, cfunc)
             check_ok(3, None, None, pyfunc, cfunc)
             check_ok(np.int8(0), np.int8(5), np.int8(1), pyfunc, cfunc)
@@ -879,7 +1001,7 @@ class TestArrayMethods(MemoryLeakMixin, TestCase):
             check_ok(0, -10, -2, np.float32)
             check_ok(0.5, 4, 2, None)
             check_ok(0, 1, 0.1, np.complex128)
-            check_ok(0, np.complex(4, 4), np.complex(1, 1), np.complex128)
+            check_ok(0, complex(4, 4), complex(1, 1), np.complex128)
             check_ok(3, 6, None, None)
             check_ok(3, None, None, None)
 
@@ -922,6 +1044,16 @@ class TestArrayMethods(MemoryLeakMixin, TestCase):
                     self.assertIn("Maximum allowed size exceeded",
                                 str(raises.exception))
 
+    def test_arange_accuracy(self):
+        # Checking arange reasonably replicates NumPy's algorithm
+        # see https://github.com/numba/numba/issues/6768
+        @jit(nopython=True)
+        def foo(step):
+            return np.arange(0, 1 + step, step)
+
+        x = 0.010101010101010102
+        self.assertPreciseEqual(foo(x), foo.py_func(x))
+
     def test_item(self):
         pyfunc = array_item
         cfunc = jit(nopython=True)(pyfunc)
@@ -1191,9 +1323,9 @@ class TestArrayMethods(MemoryLeakMixin, TestCase):
         self.assertPreciseEqual(foo(a), foo.py_func(a))
         # ndim == 2, axis == -3, BAD
         a = np.ones((1, 2))
-        with self.assertRaises(LoweringError) as raises:
+        with self.assertRaises(NumbaValueError) as raises:
             foo(a)
-        errmsg = "'axis' entry is out of bounds"
+        errmsg = "'axis' entry (-1) is out of bounds"
         self.assertIn(errmsg, str(raises.exception))
         with self.assertRaises(ValueError) as raises:
             foo.py_func(a)
@@ -1329,6 +1461,113 @@ class TestArrayMethods(MemoryLeakMixin, TestCase):
         z = x + 1j*y
         np.testing.assert_equal(pyfunc(z), cfunc(z))
 
+    def _lower_clip_result_test_util(self, func, a, a_min, a_max):
+        # verifies that type-inference is working on the return value
+        # this used to trigger issue #3489
+        def lower_clip_result(a):
+            return np.expm1(func(a, a_min, a_max))
+
+        np.testing.assert_almost_equal(
+            lower_clip_result(a),
+            jit(nopython=True)(lower_clip_result)(a))
+
+    def test_clip(self):
+        has_out = (np_clip, np_clip_kwargs, array_clip, array_clip_kwargs)
+        has_no_out = (np_clip_no_out, array_clip_no_out)
+        # TODO: scalars are not tested (issue #3469)
+        for a in (np.linspace(-10, 10, 101),
+                  np.linspace(-10, 10, 40).reshape(5, 2, 4)):
+            for pyfunc in has_out + has_no_out:
+                cfunc = jit(nopython=True)(pyfunc)
+
+                msg = "array_clip: must set either max or min"
+                with self.assertRaisesRegex(ValueError, msg):
+                    cfunc(a, None, None)
+
+                np.testing.assert_equal(pyfunc(a, 0, None), cfunc(a, 0, None))
+                np.testing.assert_equal(pyfunc(a, None, 0), cfunc(a, None, 0))
+
+                np.testing.assert_equal(pyfunc(a, -5, 5), cfunc(a, -5, 5))
+
+                if pyfunc in has_out:
+                    pyout = np.empty_like(a)
+                    cout = np.empty_like(a)
+                    np.testing.assert_equal(pyfunc(a, -5, 5, pyout),
+                                            cfunc(a, -5, 5, cout))
+                    np.testing.assert_equal(pyout, cout)
+
+                self._lower_clip_result_test_util(cfunc, a, -5, 5)
+
+    def test_clip_array_min_max(self):
+        has_out = (np_clip, np_clip_kwargs, array_clip, array_clip_kwargs)
+        has_no_out = (np_clip_no_out, array_clip_no_out)
+        # TODO: scalars are not tested (issue #3469)
+        a = np.linspace(-10, 10, 40).reshape(5, 2, 4)
+        a_min_arr = np.arange(-8, 0).astype(a.dtype).reshape(2, 4)
+        a_max_arr = np.arange(0, 8).astype(a.dtype).reshape(2, 4)
+        mins = [0, -5, a_min_arr, None]
+        maxs = [0, 5, a_max_arr, None]
+        for pyfunc in has_out + has_no_out:
+            cfunc = jit(nopython=True)(pyfunc)
+
+            for a_min in mins:
+                for a_max in maxs:
+
+                    if a_min is None and a_max is None:
+                        msg = "array_clip: must set either max or min"
+                        with self.assertRaisesRegex(ValueError, msg):
+                            cfunc(a, None, None)
+                        continue
+
+                    np.testing.assert_equal(pyfunc(a, a_min, a_max), cfunc(a, a_min, a_max))
+
+                    if pyfunc in has_out:
+                        pyout = np.empty_like(a)
+                        cout = np.empty_like(a)
+                        np.testing.assert_equal(pyfunc(a, a_min, a_max, pyout),
+                                                cfunc(a, a_min, a_max, cout))
+                        np.testing.assert_equal(pyout, cout)
+
+                    self._lower_clip_result_test_util(cfunc, a, a_min, a_max)
+
+    def test_clip_bad_array(self):
+        cfunc = jit(nopython=True)(np_clip)
+        msg = '.*The argument "a" must be array-like.*'
+        with self.assertRaisesRegex(TypingError, msg):
+            cfunc(None, 0, 10)
+
+    def test_clip_bad_min(self):
+        cfunc = jit(nopython=True)(np_clip)
+        msg = '.*The argument "a_min" must be a number.*'
+        with self.assertRaisesRegex(TypingError, msg):
+            cfunc(1, 'a', 10)
+
+    def test_clip_bad_max(self):
+        cfunc = jit(nopython=True)(np_clip)
+        msg = '.*The argument "a_max" must be a number.*'
+        with self.assertRaisesRegex(TypingError, msg):
+            cfunc(1, 1, 'b')
+
+    def test_clip_bad_out(self):
+        cfunc = jit(nopython=True)(np_clip)
+        msg = '.*The argument "out" must be an array if it is provided.*'
+        with self.assertRaisesRegex(TypingError, msg):
+            cfunc(5, 1, 10, out=6)
+
+    def test_clip_no_broadcast(self):
+        self.disable_leak_check()
+        cfunc = jit(nopython=True)(np_clip)
+        msg = ".*shape mismatch: objects cannot be broadcast to a single shape.*"
+        a = np.linspace(-10, 10, 40).reshape(5, 2, 4)
+        a_min_arr = np.arange(-5, 0).astype(a.dtype).reshape(5, 1)
+        a_max_arr = np.arange(0, 5).astype(a.dtype).reshape(5, 1)
+        min_max = [(0, a_max_arr), (-5, a_max_arr),
+                   (a_min_arr, a_max_arr),
+                   (a_min_arr, 0), (a_min_arr, 5)]
+        for a_min, a_max in min_max:
+            with self.assertRaisesRegex(ValueError, msg):
+                cfunc(a, a_min, a_max)
+
     def test_conj(self):
         for pyfunc in [array_conj, array_conjugate]:
             cfunc = jit(nopython=True)(pyfunc)
@@ -1382,7 +1621,6 @@ class TestArrayMethods(MemoryLeakMixin, TestCase):
         args = n, np.dtype('f4')
         np.testing.assert_array_equal(pyfunc(*args), cfunc(*args))
 
-
 class TestArrayComparisons(TestCase):
 
     def test_identity(self):
