@@ -4,6 +4,7 @@ import numpy as np
 
 from numba import jit, typeof
 from numba.core.compiler import compile_isolated
+from numba.np.numpy_support import numpy_version
 from numba.tests.support import TestCase, MemoryLeakMixin, tag
 import unittest
 
@@ -128,6 +129,9 @@ def array_nanpercentile_global(arr, q):
 def array_ptp_global(a):
     return np.ptp(a)
 
+def array_ptp(a):
+    return a.ptp()
+
 def array_quantile_global(arr, q):
     return np.quantile(arr, q)
 
@@ -353,7 +357,14 @@ class TestArrayReductions(MemoryLeakMixin, TestCase):
         def check(a, q, abs_tol=1e-12):
             expected = pyfunc(a, q)
             got = cfunc(a, q)
-            self.assertPreciseEqual(got, expected, abs_tol=abs_tol)
+            # NOTE: inf/nan is not checked, seems to be susceptible to upstream
+            # changes
+            finite = np.isfinite(expected)
+            if np.all(finite):
+                self.assertPreciseEqual(got, expected, abs_tol=abs_tol)
+            else:
+                self.assertPreciseEqual(got[finite], expected[finite],
+                                        abs_tol=abs_tol)
 
         a = self.random.randn(27).reshape(3, 3, 3)
         q = np.linspace(0, q_upper_bound, 14)[::-1]
@@ -411,7 +422,14 @@ class TestArrayReductions(MemoryLeakMixin, TestCase):
         def check(a, q, abs_tol=1e-14):
             expected = pyfunc(a, q)
             got = cfunc(a, q)
-            self.assertPreciseEqual(got, expected, abs_tol=abs_tol)
+            # NOTE: inf/nan is not checked, seems to be susceptible to upstream
+            # changes
+            finite = np.isfinite(expected)
+            if np.all(finite):
+                self.assertPreciseEqual(got, expected, abs_tol=abs_tol)
+            else:
+                self.assertPreciseEqual(got[finite], expected[finite],
+                                        abs_tol=abs_tol)
 
         def convert_to_float_and_check(a, q, abs_tol=1e-14):
             expected = pyfunc(a, q).astype(np.float64)
@@ -440,13 +458,16 @@ class TestArrayReductions(MemoryLeakMixin, TestCase):
         q = np.array(1)
         _check(a, q)
 
-        a = True
-        q = False
-        _check(a, q)
+        if numpy_version < (1, 20):
+            # NumPy 1.20+ rewrites the interpolation part of percentile/quantile
+            # to use np.subtract which doesn't support bools.
+            a = True
+            q = False
+            _check(a, q)
 
-        a = np.array([False, True, True])
-        q = a
-        _check(a, q)
+            a = np.array([False, True, True])
+            q = a
+            _check(a, q)
 
         a = 5
         q = q_upper_bound / 2
@@ -665,8 +686,11 @@ class TestArrayReductions(MemoryLeakMixin, TestCase):
         np.random.shuffle(arr)
         self.assertPreciseEqual(cfunc(arr), pyfunc(arr))
         # Test with a NaT
-        arr[arr.size // 2] = 'NaT'
-        self.assertPreciseEqual(cfunc(arr), pyfunc(arr))
+        if numpy_version != (1, 21) and 'median' not in pyfunc.__name__:
+            # There's problems with NaT handling in "median" on at least NumPy
+            # 1.21.{3, 4}. See https://github.com/numpy/numpy/issues/20376
+            arr[arr.size // 2] = 'NaT'
+            self.assertPreciseEqual(cfunc(arr), pyfunc(arr))
         if 'median' not in pyfunc.__name__:
             # Test with (val, NaT)^N (and with the random NaT from above)
             # use a loop, there's some weird thing/bug with arr[1::2] = 'NaT'
@@ -797,6 +821,17 @@ class TestArrayReductions(MemoryLeakMixin, TestCase):
         for a in a_variations():
             check(a)
 
+    def test_ptp_method(self):
+        # checks wiring of np.ndarray.ptp() only, `np.ptp` test above checks
+        # the actual alg
+        pyfunc = array_ptp
+        cfunc = jit(nopython=True)(pyfunc)
+
+        a = np.arange(10)
+        expected = pyfunc(a)
+        got = cfunc(a)
+        self.assertPreciseEqual(expected, got)
+
     def test_ptp_complex(self):
         pyfunc = array_ptp_global
         cfunc = jit(nopython=True)(pyfunc)
@@ -924,6 +959,150 @@ class TestArrayReductions(MemoryLeakMixin, TestCase):
             for a in a_variations():
                 check(a)
 
+    def test_argmax_axis_1d_2d_4d(self):
+        arr1d = np.array([0, 20, 3, 4])
+        arr2d = np.arange(6).reshape(2, 3)
+        arr2d[0,1] += 100
+
+        arr4d = np.arange(120).reshape(2, 3, 4, 5) + 10
+        arr4d[0, 1, 1, 2] += 100
+        arr4d[1, 0, 0, 0] -= 51
+
+        for arr in [arr1d, arr2d, arr4d]:
+            axes = list(range(arr.ndim)) + [
+                -(i+1) for i in range(arr.ndim)
+            ]
+            py_functions = [
+                lambda a, _axis=axis: np.argmax(a, axis=_axis)
+                for axis in axes
+            ]
+            c_functions = [
+                jit(nopython=True)(pyfunc) for pyfunc in py_functions
+            ]
+            for cfunc in c_functions:
+                self.assertPreciseEqual(cfunc.py_func(arr), cfunc(arr))
+
+    def test_argmax_axis_out_of_range(self):
+        arr1d = np.arange(6)
+        arr2d = np.arange(6).reshape(2, 3)
+
+        @jit(nopython=True)
+        def jitargmax(arr, axis):
+            return np.argmax(arr, axis)
+
+        def assert_raises(arr, axis):
+            with self.assertRaisesRegex(ValueError, "axis.*out of bounds"):
+                jitargmax.py_func(arr, axis)
+            with self.assertRaisesRegex(ValueError, "axis.*out of bounds"):
+                jitargmax(arr, axis)
+
+        assert_raises(arr1d, 1)
+        assert_raises(arr1d, -2)
+        assert_raises(arr2d, -3)
+        assert_raises(arr2d, 2)
+
+    def test_argmax_axis_must_be_integer(self):
+        arr = np.arange(6)
+
+        @jit(nopython=True)
+        def jitargmax(arr, axis):
+            return np.argmax(arr, axis)
+
+        with self.assertTypingError() as e:
+            jitargmax(arr, "foo")
+        self.assertIn("axis must be an integer", str(e.exception))
+
+    def test_argmax_method_axis(self):
+        arr2d = np.arange(6).reshape(2, 3)
+
+        def argmax(arr):
+            return arr2d.argmax(axis=0)
+
+        self.assertPreciseEqual(argmax(arr2d),
+                                jit(nopython=True)(argmax)(arr2d))
+
+    def test_argmax_return_type(self):
+        # See issue #7853, return type should be intp not based on input type
+        arr2d = np.arange(6, dtype=np.uint8).reshape(2, 3)
+
+        def argmax(arr):
+            return arr2d.argmax(axis=0)
+
+        self.assertPreciseEqual(argmax(arr2d),
+                                jit(nopython=True)(argmax)(arr2d))
+
+    def test_argmin_axis_1d_2d_4d(self):
+        arr1d = np.array([0, 20, 3, 4])
+        arr2d = np.arange(6).reshape(2, 3)
+        arr2d[0,1] += 100
+
+        arr4d = np.arange(120).reshape(2, 3, 4, 5) + 10
+        arr4d[0, 1, 1, 2] += 100
+        arr4d[1, 0, 0, 0] -= 51
+
+        for arr in [arr1d, arr2d, arr4d]:
+            axes = list(range(arr.ndim)) + [
+                -(i+1) for i in range(arr.ndim)
+            ]
+            py_functions = [
+                lambda a, _axis=axis: np.argmin(a, axis=_axis)
+                for axis in axes
+            ]
+            c_functions = [
+                jit(nopython=True)(pyfunc) for pyfunc in py_functions
+            ]
+            for cfunc in c_functions:
+                self.assertPreciseEqual(cfunc.py_func(arr), cfunc(arr))
+
+    def test_argmin_axis_out_of_range(self):
+        arr1d = np.arange(6)
+        arr2d = np.arange(6).reshape(2, 3)
+
+        @jit(nopython=True)
+        def jitargmin(arr, axis):
+            return np.argmin(arr, axis)
+
+        def assert_raises(arr, axis):
+            with self.assertRaisesRegex(ValueError, "axis.*out of bounds"):
+                jitargmin.py_func(arr, axis)
+            with self.assertRaisesRegex(ValueError, "axis.*out of bounds"):
+                jitargmin(arr, axis)
+
+        assert_raises(arr1d, 1)
+        assert_raises(arr1d, -2)
+        assert_raises(arr2d, -3)
+        assert_raises(arr2d, 2)
+
+    def test_argmin_axis_must_be_integer(self):
+        arr = np.arange(6)
+
+        @jit(nopython=True)
+        def jitargmin(arr, axis):
+            return np.argmin(arr, axis)
+
+        with self.assertTypingError() as e:
+            jitargmin(arr, "foo")
+        self.assertIn("axis must be an integer", str(e.exception))
+
+    def test_argmin_method_axis(self):
+        arr2d = np.arange(6).reshape(2, 3)
+
+        def argmin(arr):
+            return arr2d.argmin(axis=0)
+
+        self.assertPreciseEqual(argmin(arr2d),
+                                jit(nopython=True)(argmin)(arr2d))
+
+    def test_argmin_return_type(self):
+        # See issue #7853, return type should be intp not based on input type
+        arr2d = np.arange(6, dtype=np.uint8).reshape(2, 3)
+
+        def argmin(arr):
+            return arr2d.argmin(axis=0)
+
+        self.assertPreciseEqual(argmin(arr2d),
+                                jit(nopython=True)(argmin)(arr2d))
+
     @classmethod
     def install_generated_tests(cls):
         # These form a testing product where each of the combinations are tested
