@@ -15,7 +15,8 @@ import numpy as np
 from numba import jit
 from numba.core import types, utils
 import unittest
-from numba.tests.support import TestCase, tag, CompilationCache
+from numba.tests.support import (TestCase, tag, CompilationCache,
+                                 skip_unless_py10_or_later)
 
 from numba.cpython.unicode import compile_time_get_string_data
 from numba.cpython import hashing
@@ -54,7 +55,11 @@ class TestHashingSetup(TestCase):
         with warnings.catch_warnings(record=True) as warns:
             # Cause all warnings to always be triggered.
             warnings.simplefilter("always")
-            import numba
+            from numba import njit
+            @njit
+            def foo():
+                hash(1)
+            foo()
             assert len(warns) > 0
             expect = "FNV hashing is not implemented in Numba. See PEP 456"
             for w in warns:
@@ -112,9 +117,14 @@ class BaseTest(TestCase):
                 yield a + a.mean()
 
         # Infs, nans, zeros, magic -1
-        a = typ([0.0, 0.5, -0.0, -1.0, float('inf'), -float('inf'),
-                 float('nan')])
-        yield a
+        a = [0.0, 0.5, -0.0, -1.0, float('inf'), -float('inf'),]
+
+        # Python 3.10 has a hash for nan based on the pointer to the PyObject
+        # containing the nan, skip this input and use explicit test instead.
+        if utils.PYVERSION < (3, 10):
+            a.append(float('nan'))
+
+        yield typ(a)
 
     def complex_samples(self, typ, float_ty):
         for real in self.float_samples(float_ty):
@@ -123,7 +133,13 @@ class BaseTest(TestCase):
                 real = real[:len(imag)]
                 imag = imag[:len(real)]
                 a = real + typ(1j) * imag
-                yield a
+                # Python 3.10 has a hash for nan based on the pointer to the
+                # PyObject containing the nan, skip input that ends up as nan
+                if utils.PYVERSION >= (3, 10):
+                    if not np.any(np.isnan(a)):
+                        yield a
+                else:
+                    yield a
 
 
 class TestNumberHashing(BaseTest):
@@ -207,6 +223,20 @@ class TestNumberHashing(BaseTest):
         self.check_hash_values([np.int32(-0x7fffff9c)])
 
 
+    @skip_unless_py10_or_later
+    def test_py310_nan_hash(self):
+        # On Python 3.10+ nan's hash to a value which is based on the pointer to
+        # the PyObject containing the nan. Numba cannot replicate as there's no
+        # object, it instead produces equivalent behaviour, i.e. hashes to
+        # something "unique".
+
+        # Run 10 hashes, make sure that the "uniqueness" is sufficient that
+        # there's more than one hash value. Not much more can be done!
+        x = [float('nan') for i in range(10)]
+        out = set([self.cfunc(z) for z in x])
+        self.assertGreater(len(out), 1)
+
+
 class TestTupleHashing(BaseTest):
     """
     Test hashing of tuples.
