@@ -17,10 +17,10 @@ from numba.core.extending import overload
 
 
 enable_pyobj_flags = Flags()
-enable_pyobj_flags.set("enable_pyobject")
+enable_pyobj_flags.enable_pyobject = True
 
 force_pyobj_flags = Flags()
-force_pyobj_flags.set("force_pyobject")
+force_pyobj_flags.force_pyobject = True
 
 Point = namedtuple('Point', ('a', 'b'))
 
@@ -312,6 +312,10 @@ def list_mul(n, v):
     a = list(range(n))
     return a * v
 
+def list_mul2(n, v):
+    a = list(range(n))
+    return v * a
+
 def list_mul_inplace(n, v):
     a = list(range(n))
     a *= v
@@ -656,6 +660,9 @@ class TestLists(MemoryLeakMixin, TestCase):
     def test_mul(self):
         self.check_mul(list_mul)
 
+    def test_mul2(self):
+        self.check_mul(list_mul2)
+
     def test_mul_inplace(self):
         self.check_mul(list_mul_inplace)
 
@@ -920,6 +927,15 @@ class TestListManagedElements(ManagedListTestCase):
 
         self._check_element_equal(pyfunc)
 
+    def test_reflect_insert(self):
+        """make sure list.insert() doesn't crash for refcounted objects (see #7553)
+        """
+
+        def pyfunc(con):
+            con.insert(1, np.arange(4).astype(np.intp))
+
+        self._check_element_equal(pyfunc)
+
     def test_append(self):
         def pyfunc():
             con = []
@@ -1766,7 +1782,7 @@ class TestLiteralLists(MemoryLeakMixin, TestCase):
                 l = ['a', 1, 2j]
             else:
                 l = ['b', 2]
-            return l[0]
+            return l[0], l[1], l[0], l[1] # defeat py310 inliner
 
         with self.assertRaises(errors.TypingError) as raises:
             foo(100)
