@@ -13,7 +13,7 @@ import unittest
 from numba.np import numpy_support
 
 enable_pyobj_flags = Flags()
-enable_pyobj_flags.set("enable_pyobject")
+enable_pyobj_flags.enable_pyobject = True
 
 no_pyobj_flags = Flags()
 
